/*
 * Copyright (C) Vladimir Oleynik <dzo@simtreas.ru> 2015-2020
 * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
*/

#include "bsed.h"
#include <stdarg.h>

/*
 * process_escape_sequence()
 *
 * Copyright (C) Manuel Novoa III <mjn3@codepoet.org>
 * and Vladimir Oleynik <dzo@simtreas.ru>
 *
 */

#define WANT_HEX_ESCAPES 1

/* Usual "this only works for ascii compatible encodings" disclaimer. */
#undef _tolower
#define _tolower(X) ((X)|((char) 0x20))

char process_escape_sequence(const char **ptr)
{
	static const char charmap[] = {
		'a',  'b',  'e',     'f',  'n',  'r',  't',  'v', '\\', 0,
		'\a', '\b', '\033', '\f', '\n', '\r', '\t', '\v', '\\', '\\' };

	const char *p;
	const char *q;
	unsigned int num_digits;
	unsigned int r;
	unsigned int n;
	unsigned int d;
	unsigned int base;

	num_digits = n = 0;
	base = 8;
	q = *ptr;

#ifdef WANT_HEX_ESCAPES
	if (*q == 'x') {
		++q;
		base = 16;
		++num_digits;
	}
#endif

	do {
		d = (unsigned int)(*q - '0');
#ifdef WANT_HEX_ESCAPES
		if (d >= 10) {
			d = ((unsigned int)(_tolower(*q) - 'a')) + 10;
		}
#endif

		if (d >= base) {
#ifdef WANT_HEX_ESCAPES
			if ((base == 16) && (!--num_digits)) {
/*                              return '\\'; */
				--q;
			}
#endif
			break;
		}

		r = n * base + d;
		if (r > 255) {
			break;
		}

		n = r;
		++q;
	} while (++num_digits < 3);

	if (num_digits == 0) {  /* mnemonic escape sequence? */
		p = charmap;
		do {
			if (*p == *q) {
				q++;
				break;
			}
		} while (*++p);
		n = *(p+(sizeof(charmap)/2));
	}

	*ptr = q;

	return (char) n;
}

void error_msg(int errc, const char *s, ...)
{
	va_list p;

	va_start(p, s);
	fprintf(stderr, "%s: ", prg_name);
	vfprintf(stderr, s, p);
	va_end(p);
	putc('\n', stderr);
	exit(errc);
}

static inline void *tst_null(void *s, size_t sz)
{
	if(s == NULL && sz != 0)
		error_msg(1, " ");
	return s;
}

void *xmalloc(size_t sz)
{
	void *b = malloc(sz);
	return tst_null(b, sz);
}

void *xrealloc(void *b, size_t sz)
{
	b = realloc(b, sz);
	return tst_null(b, sz);
}

void *xcalloc(size_t n, size_t sz)
{
	void *b = calloc(n, sz);
	return tst_null(b, sz);
}
