/* WARNING! POSIX need int, but bsed need ssize_t */
typedef int regoff_t;

typedef struct {
  size_t re_nsub;  /* Number of parenthesized subexpressions. */
  void *value;     /* For internal use only. */
} regex_t;

typedef struct {
  regoff_t rm_so;
  regoff_t rm_eo;
} regmatch_t;


typedef enum {
  REG_OK = 0,           /* No error. */
  /* POSIX tre_regcomp() return error codes.  (In the order listed in the
     standard.)  */
  REG_NOMATCH,          /* No match. */
  REG_BADPAT,           /* Invalid regexp. */
  REG_ECOLLATE,         /* Unknown collating element. */
  REG_ECTYPE,           /* Unknown character class name. */
  REG_EESCAPE,          /* Trailing backslash. */
  REG_ESUBREG,          /* Invalid back reference. */
  REG_EBRACK,           /* "[]" imbalance */
  REG_EPAREN,           /* "\(\)" or "()" imbalance */
  REG_EBRACE,           /* "\{\}" or "{}" imbalance */
  REG_BADBR,            /* Invalid content of {} */
  REG_ERANGE,           /* Invalid use of range operator */
  REG_ESPACE,           /* Out of memory.  */
  REG_BADRPT            /* Invalid use of repetition operators. */
} reg_errcode_t;

/* POSIX regcomp() flags. */
#include "opt_gen.h"
BITS_GEN(REG, EXTENDED, ICASE, NEWLINE, NOSUB)
#include "opt_gen.h"

/* POSIX tre_regexec() flags. */
#define REG_NOTBOL 1
#define REG_NOTEOL (REG_NOTBOL << 1)

/* Extra tre_regexec() flags. */
#define REG_APPROX_MATCHER       (REG_NOTEOL << 1)

/* The POSIX.2 regexp functions */
extern int regcomp(regex_t *preg, const char *regex, int cflags);

extern int regexec(const regex_t *preg, const char *string, size_t nmatch, regmatch_t pmatch[], int eflags);

extern size_t regerror(int errcode, const regex_t *preg, char *errbuf, size_t errbuf_size);

extern void regfree(regex_t *preg);


/* Versions with a maximum length argument and therefore the capability to
   handle null characters in the middle of the strings (not in POSIX.2). */
extern int tre_regncomp(regex_t *preg, const char *regex, size_t len, int cflags);

extern int tre_regnexec(const regex_t *preg, const char *string, size_t len, size_t nmatch, regmatch_t pmatch[], int eflags);
